@@ -1,5 +1,7 @@
 
-from keywords import DiffKeywords
+from .keywords import DiffKeywords
+
 
 class DiffLibrary(DiffKeywords):
     ROBOT_LIBRARY_SCOPE = 'GLOBAL'
+
