@@ -1,44 +1,54 @@
-import sys
-import os
-
-import subprocess
-
+#!/usr/bin/env python
 
 from robot.libraries.BuiltIn import BuiltIn
+import os
+import subprocess
+import sys
+import tempfile
+
+from DiffLibrary.compat import PY3
 
 class DiffKeywords(object):
+    """
+    *** Settings ***
+    Library   DiffLibrary
+    *** Testcases ***
+    Diff Two Identical Files
+        Diff Files  loremipsum.txt  loremipsum.txt
+    """
     ROBOT_LIBRARY_SCOPE = 'Global'
-
     root = os.path.abspath(os.path.join(__file__, '..'))
 
-
     def __init__(self):
         self.builtin = BuiltIn()
 
-
     def _getdiff(self):
-        ''' returns path to diff depending on platform '''
+        '''
+        returns path to diff depending on platform
+        '''
 
         diff = 'diff --strip-trailing-cr'
-        if 'win' in sys.platform:
-            diff = "%s --strip-trailing-cr " %os.path.join(self.root, 'bin', 'diff', 'diff.exe')
+        if 'win32' in sys.platform:
+            diff = "%s --strip-trailing-cr " % os.path.join(self.root, 'bin',
+                                                            'diff', 'diff.exe')
         return diff
 
-    
     def _getndiff(self):
         raise Exception("Not implemented")
 
+    def _newdiff(self, actfile, reffile, diff_func='_getdiff',
+                     diff_params=None, diff_file_path=None, filter=None):
+        '''
+        compare two sxv4dump files
 
-    def _newdiff(self, actfile, reffile, diff_func='_getdiff', diff_params=None, diff_file_path=None, filter=None):
-        ''' compare two sxv4dump files
+        ``actfile:`` sxv4dump file created in latest build
 
-        *actfile:* sxv4dump file created in latest build
+        ``reffile:`` reference sxv4dump file, gzipped
 
-        *reffile:* reference sxv4dump file, gzipped
+        ``diff_file_path:`` absolute path of directory to save the diff delta
 
-        *diff_file_path:* absolute path of directory to save the diff delta
-
-        *diff_cmd:* custom diff command'''
+        ``diff_cmd:`` custom diff command
+        '''
 
         diff_types = {'_getdiff': self._getdiff, '_getndiff': self._getndiff}
         diff_lines = {'_getdiff': 2, '_getndiff': 4}
@@ -101,7 +111,8 @@ class DiffKeywords(object):
         # construct the diff command
         diff_function = diff_types.get(diff_func, '_getdiff')
         if diff_params != None:
-            diff_cmd = '%s %s "%s" "%s"' %(diff_function(), diff_params, actfile, reffile)
+            diff_cmd = '%s %s "%s" "%s"' % (
+            diff_function(), diff_params, actfile, reffile)
         else:
             diff_cmd = '%s "%s" "%s"' %(diff_function(), actfile, reffile)
         output, rc = self._run(diff_cmd)
@@ -126,20 +137,20 @@ class DiffKeywords(object):
         # because the filter will remove the timestap diff's
         lines_to_skip = diff_lines.get(diff_func, '_getdiff')
         if lines and len(lines) > lines_to_skip:
-            print '\n'.join(lines)
-            self.builtin.fail("differences found between %s and %s" %(actfile, reffile))
-
-
+            print('\n'.join(lines))
+            self.builtin.fail(
+                "differences found between %s and %s" % (actfile, reffile))
 
     def diff_files(self, file1, file2, fail=True):
         ''' Diff two text files
 
-        `file1`: absolute path to the first first file
+        ``file1``: absolute path to the first first file
 
-        `file2`: absolute path to the first second file
+        ``file2``: absolute path to the first second file
 
-        `fail`:  If there are differences it will throw an exception and test will fail
-                 defaults to True, if False test's will continue '''
+        ``fail``:  If there are differences it will throw an exception and
+        test will fail defaults to True, if False test's will continue
+        '''
 
         self.builtin.log("file1: %s" %file1)
         self.builtin.log("file2: %s" %file2)
@@ -150,7 +161,7 @@ class DiffKeywords(object):
         else:
             try:
                 self._newdiff(file1, file2)
-            except Exception, e:
+            except Exception as e:
                 self.builtin.log(e)
 
 
@@ -162,16 +173,52 @@ class DiffKeywords(object):
         self.builtin.log(cmd)
         #cmd = process_cmd(cmd)
 
-        # run the given command in a child shell process (cmd.exe on win and sh/bash on *nix)
-        self.cmd = subprocess.Popen(cmd + ' 2>&1', shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+        # run the given command in a child shell process
+        # (cmd.exe on win and sh/bash on *nix)
+        self.cmd = subprocess.Popen(cmd + ' 2>&1', shell=True,
+                                    stdout=subprocess.PIPE,
+                                    stderr=subprocess.STDOUT)
 
         # it will block here and try to read everything into memory
         output = self.cmd.communicate()[0]
 
+        if PY3:
+            if isinstance(output, bytes):
+                output = output.decode(encoding='utf-8')
+
         return output, self.cmd.wait()
 
+    def diff_outputs(self, text1, text2, fail=True):
+        ''' Diff between target and source texts
 
+        ``text1``: Target output
+
+        ``text2``: Source output
+
+        ``fail``:  If there are differences it will throw an exception and
+        test will fail defaults to True, if False test's will continue
+        '''
+        file1 = tempfile.NamedTemporaryFile(mode='w', suffix='.txt',
+                                            delete=False)
+        file1.write(text1)
+        file1.close()
+        file2 = tempfile.NamedTemporaryFile(mode='w', suffix='.txt',
+                                            delete=False)
+        file2.write(text2)
+        file2.close()
+
+        self.builtin.log("file1: %s" % file1.name)
+        self.builtin.log("file2: %s" % file2.name)
+
+        fail = self.builtin.convert_to_boolean(fail)
+        if fail:
+            self._newdiff(file1.name, file2.name)
+        else:
+            try:
+                self._newdiff(file1, file2)
+            except Exception as e:
+                self.builtin.log(e)
+
+        os.remove(file1.name)
+        os.remove(file2.name)
 
-if __name__ == "__main__":
-    d = DiffKeywords()
-    print d.getdiff()
